%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Academic DG code for the linear advection equation
% Ruben Sevilla
% Zienkiewicz Centre for Computational Engineering, Swansea University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clearvars
close all
clc
setpath

% Spatial discretisation---------------------------------------------------
degree = 2;
meshFile = sprintf('meshRectangle_H1_P%d', degree);
load(meshFile)

% Time discretisation------------------------------------------------------
tEnd = 4;

% Problem parameters ------------------------------------------------------
problemParams.inflowBC = @convectionFunctionDirichlet;
problemParams.a = [1 0];
problemParams.faces = @convectionFacesTerm;
problemParams.elems = @convectionDivergenceTerm;

disp('Preprocess_________________________________________________________')   
h = computeMinElementSize(X, T);
[intFaces, extFaces] = GetFaces(T(:,1:3));

% Time step
normA = norm(problemParams.a);
Ct = 0.2;
dt = Ct*h/(normA*degree^2);

% Reference element information -------------------------------------------
referenceElement = createReferenceElementTri(degree);
[nOfElements,nOfElemNodes] = size(T);
nOfFaceNodes = degree + 1;
[M,C,m] = referenceElementMatrices(referenceElement, nOfElemNodes, nOfFaceNodes);

disp('Initialisation_____________________________________________________')
nOfCOmponents = 1;
u = zeros(nOfElemNodes,nOfCOmponents,nOfElements);
t = 0;

disp('Time integration___________________________________________________')
[u,t] = temporalIntegration(X, T, u, referenceElement, M, C, m,...
    t, dt, tEnd, intFaces, extFaces, problemParams);

disp('Postprocess________________________________________________________')
u = reshape(u, numel(u), 1);
plotDiscontinuosSolution(X,T,u,referenceElement,10)

% analytical solution
u_ex = @convectionFunctionDirichlet;

%Error for the DG solution
Error = computeL2Norm(referenceElement,X,T,u,u_ex,t);
fprintf('Error DG = %e\n',Error);
disp(' ')