%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Academic DG code for the linear advection equation
% Ruben Sevilla
% Zienkiewicz Centre for Computational Engineering, Swansea University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clearvars
close all
clc
setpath

% Spatial discretisation---------------------------------------------------
degree = 5;
load(['mesh2_P',num2str(degree)]); 

% Time discretisation------------------------------------------------------
tEnd = 1;

% Problem parameters ------------------------------------------------------
problemParams.eps = 1;
problemParams.mu = 1;
problemParams.faces = @maxwellFacesTerm;
problemParams.elems = @maxwellDivergenceTerm;

disp('Preprocess_________________________________________________________')   
h = computeMinElementSize(X, T);
[intFaces, extFaces] = GetFaces(T(:,1:3));

% Time step
c = 1/sqrt(problemParams.eps*problemParams.mu);
Ct = 0.2;
dt = Ct*h/(c*degree^2);

% Reference element information -------------------------------------------
referenceElement = createReferenceElementTri(degree);
[nOfElements,nOfElemNodes] = size(T);
nOfFaceNodes = degree + 1;
[M,C,m] = referenceElementMatrices(referenceElement, nOfElemNodes, nOfFaceNodes);

disp('Initialisation_____________________________________________________')
[U, t] = maxwellInitialCondition(X, T, nOfElemNodes,nOfElements);

disp('Time integration___________________________________________________')
[U,t] = temporalIntegration(X, T, U, referenceElement, M, C, m,...
    t, dt, tEnd, intFaces, extFaces, problemParams);

disp('Postprocess________________________________________________________')
u1 = reshape(U(:,1,:), numel(U(:,1,:)), 1);
u2 = reshape(U(:,2,:), numel(U(:,2,:)), 1);
u3 = reshape(U(:,3,:), numel(U(:,3,:)), 1);
figure(1), clf
plotDiscontinuosSolution(X,T,u1,referenceElement)
figure(2), clf
plotDiscontinuosSolution(X,T,u2,referenceElement)
figure(3), clf
plotDiscontinuosSolution(X,T,u3,referenceElement)
