function F = maxwellDivergenceTerm(X,T,U,M,C,F,problemParams)

[nOfElemNodes, nOfComponents, nOfElements] = size(U);
Fx = zeros(nOfElemNodes, nOfComponents);
Fy = zeros(nOfElemNodes, nOfComponents);

for iElem = 1:nOfElements
    % Jacobian
    Te = T(iElem,:);
    Xe = X(Te,:);
    J = [(Xe(2,:)-Xe(1,:))/2 ; (Xe(3,:)-Xe(1,:))/2];
    invJ = inv(J);
    
    % Hyperbolic fluxes (TE mode)
    Fx(:,2) =   (1/problemParams.mu) *U(:,3,iElem);
    Fx(:,3) =   (1/problemParams.eps)*U(:,2,iElem);
    Fy(:,1) =  -(1/problemParams.mu) *U(:,3,iElem);
    Fy(:,3) =  -(1/problemParams.eps)*U(:,1,iElem);
    
    % Convection matrices
    Cx = invJ(1,1)*C(:,:,1) + invJ(1,2)*C(:,:,2);
    Cy = invJ(2,1)*C(:,:,1) + invJ(2,2)*C(:,:,2);
    
    % ODE residual
    F(:,:,iElem) = M\( F(:,:,iElem)/det(J) + Cx*Fx + Cy*Fy );
end
