function F = maxwellFacesTerm(X, T, u, referenceElement, m, t, intFaces, extFaces, problemParams)

% Initialization
F = zeros(size(u));

nOfInterior = size(intFaces,1);
for i = 1:nOfInterior
    iElemL = intFaces(i,1);
    iFaceL = intFaces(i,2);
    iElemR = intFaces(i,3);
    iFaceR = intFaces(i,4);
    nodesFaceL = referenceElement.faceNodes(iFaceL,:);
    nodesFaceR = referenceElement.faceNodes(iFaceR,end:-1:1);
    
    % Outward unit normal and face length
    Te = T(iElemL,:);
    Xe = X(Te,:);
    Xf = Xe(nodesFaceL,:);
    w = Xf(end,:)-Xf(1,:);
    L = norm(w);
    n = [w(2),-w(1)]/L;
    
    % Left and right states
    uL = u(nodesFaceL, :, iElemL);
    uR = u(nodesFaceR, :, iElemR);
    
    % Left element----------------------------------
    anMinusJumpU = maxwellAnMinusJumpU(uL, uR, n, problemParams);
    F(nodesFaceL,:,iElemL) = F(nodesFaceL,:,iElemL) + m*anMinusJumpU*L/2;
    
    % Right element---------------------------------
    anMinusJumpU = maxwellAnMinusJumpU(uR, uL, -n, problemParams);
    F(nodesFaceR,:,iElemR) = F(nodesFaceR,:,iElemR) + m*anMinusJumpU*L/2;
end

nOfExterior = size(extFaces,1);
for i = 1:nOfExterior
    iElemL = extFaces(i,1);
    iFaceL = extFaces(i,2);
    nodesFaceL = referenceElement.faceNodes(iFaceL,:);
    
    % Outward unit normal and face length
    Te = T(iElemL,:);
    Xe = X(Te,:);
    Xf = Xe(nodesFaceL,:);
    w = Xf(end,:)-Xf(1,:);
    L = norm(w);
    n = [w(2),-w(1)]/L;
    
    % Left and right states
    uL = u(nodesFaceL, :, iElemL);
    uR = zeros(size(uL));
    
    % Left element---------------------------------
    anMinusJumpU = maxwellAnMinusJumpU(uL, uR, n, problemParams);
    F(nodesFaceL,:,iElemL) = F(nodesFaceL,:,iElemL) + m*anMinusJumpU*L/2;
end