function [u,t] = temporalIntegration(X,T,u,referenceElement,M,C,m,...
    t,dt,tEnd,intFaces,extFaces,problemParams)

nStep = round(tEnd/dt);
nMessage = round(nStep/20);
iCompPlot = 1;

for iStep = 1:nStep
    % RK4
    F1 = hyperbolicF(X,T,u,referenceElement,M,C,m,t,intFaces,extFaces,problemParams);
    uTmp = u - (dt/2)*F1;
    t = t + dt/2;
    F2 = hyperbolicF(X,T,uTmp,referenceElement,M,C,m,t,intFaces,extFaces,problemParams);
    uTmp = u - (dt/2)*F2;
    F3 = hyperbolicF(X,T,uTmp,referenceElement,M,C,m,t,intFaces,extFaces,problemParams);
    uTmp = u - dt*F3;
    t = iStep*dt;
    F4 = hyperbolicF(X,T,uTmp,referenceElement,M,C,m,t,intFaces,extFaces,problemParams);
    u = u - (dt/6)*(F1+2*F2+2*F3+F4); 

    if mod(iStep,nMessage)==0
        fprintf('Step %d of %d\n',iStep,nStep)        
        uPlot = reshape(u(:,iCompPlot,:), numel(u(:,iCompPlot,:)), 1);
        plotDiscontinuosSolution(X,T,uPlot,referenceElement)
        view(3)
        pause(0.05)
        clf
    end
end


function F = hyperbolicF(X,T,u,referenceElement,M,C,m,...
    t,intFaces,extFaces,problemParams)

F = problemParams.faces(X,T,u,referenceElement,m,t,intFaces,extFaces,problemParams);
F = problemParams.elems(X,T,u,M,C,F,problemParams);

