function [KK f localSolverMat localSolverVec] = hdgMatrixOseen(viscosityMatrix,advectionField,bodyForce,X,T,F,referenceElement,infoFaces,tau)
% [KK f localSolverMat localSolverVec] = hdgMatrixOseen(viscosityMatrix,advectionField,bodyForce,X,T,F,referenceElement,infoFaces,tau)
% The geometry is approximated with the degree given by the mesh (X,T) and the
% reference element in referenceElement
% Nodal values are assumed to be ordered by components: u = [...u1... ...u2...]

nOfFaces = max(max(F));
nOfElements = size(T,1);
nOfInteriorFaces = size(infoFaces.intFaces,1);
nOfFaceNodes = size(referenceElement.NodesCoord1d,1);
aux=nOfFaceNodes:-1:1; aux=[aux,aux+nOfFaceNodes]; 
indflip=[aux,2*nOfFaceNodes+aux,4*nOfFaceNodes+aux];
f = zeros(2*nOfFaces*nOfFaceNodes+nOfElements,1);
localSolverMat = cell(nOfElements,1);
localSolverVec = cell(nOfElements,1);

% loop in elements
nOfuhatNodes=2*nOfFaces*nOfFaceNodes;
indK = 1; indf=1; n = nOfElements*(1+2*3*nOfFaceNodes)^2; ind_i  = zeros(1,n); ind_j  = zeros(1,n); coef_K = zeros(1,n);
for iElem = 1:nOfElements
    Te = T(iElem,:);
    Xe = X(Te,:);
    Fe = F(iElem,:);
    isFeInterior = (Fe <= nOfInteriorFaces); %Boolean (1=interior face)

    % elemental matrices
    [localSolverMatElem,localSolverVecElem,Alu,AlL,Alp,All,Arl] = KKeElementalMatricesSuperParametric(viscosityMatrix,advectionField,bodyForce,Xe,referenceElement,isFeInterior,tau(iElem,:));
    
    % Interior faces seen from the second element are flipped to have
    % proper orientation
    flipFace = zeros(1,3); %Boolean (1=face to be flipped)
    flipFace(isFeInterior)=any(infoFaces.intFaces(Fe(isFeInterior),[1 3])<iElem,2);
    indL=1:6*nOfFaceNodes; 
    aux=ones(1,2*nOfFaceNodes); aux = [flipFace(1)*aux, flipFace(2)*aux, flipFace(3)*aux];
    indL(aux==1)=indflip(aux==1); %permutation for local numbering
    indr = length(indL)+1; %new component in Gamma_e for pressure average 
        
    localSolverMatElem=localSolverMatElem(:,[indL,indr]);
    AlL=AlL(indL,:);  Alu=Alu(indL,:); Alp=Alp(indL,:); All=All(indL,indL);
    Arl=Arl(:,indL);
    
    %The local problem solver is stored for postprocess
    localSolverMat{iElem} = sparse(localSolverMatElem);
    localSolverVec{iElem} = sparse(localSolverVecElem);
    
    %Elemental matrices to be assembled
    KKe = [Alu AlL Alp]*localSolverMatElem + [All,zeros(2*3*nOfFaceNodes,1)];
    KKe = [KKe ; [Arl 0]];
    ffe = -[Alu AlL Alp]*localSolverVecElem;
    ffe = [ffe ; 0];
    
    aux = (1:2*nOfFaceNodes);
    indrow = [(Fe(1)-1)*2*nOfFaceNodes + aux,(Fe(2)-1)*2*nOfFaceNodes + aux,(Fe(3)-1)*2*nOfFaceNodes + aux];
    indrow = [indrow, nOfuhatNodes+iElem]; %last component of Lambda_e=rho_e
    indcol = indrow;
    f(indrow) = f(indrow) + ffe;
    %KK(indrow,indcol)=KK(indrow,indcol)+ KKe;
    for irow = 1:(6*nOfFaceNodes)+1;
        for icol = 1:(6*nOfFaceNodes)+1 %last component of Lambda_e=rho_e
            ind_i(indK)  = indrow(irow); ind_j(indK)  = indcol(icol);
            coef_K(indK) = KKe(irow,icol); 
            indK = indK+1;
        end
    end
end
KK = sparse(ind_i,ind_j,coef_K);


%%
%% ELEMENTAL MATRIX
function [LocalSolverMatElem,LocalSolverVecElem,Alu,AlL,Alp,All,Arl] = KKeElementalMatricesSuperParametric(viscosityMatrix,advectionField,bodyForce,Xe,referenceElement,isInterior,tau)

nOfElementNodes = size(referenceElement.NodesCoord,1);
nOfFaceNodes = size(referenceElement.NodesCoord1d,1);
faceNodes = referenceElement.faceNodes;
nOfFaces = 3; %triangles

% Information of the reference element
N = referenceElement.N;
Nxi = referenceElement.Nxi; Neta = referenceElement.Neta;
N1d = referenceElement.N1d; Nx1d = referenceElement.N1dxi;
%Numerical quadrature
IPw_f = referenceElement.IPweights1d; ngf = length(IPw_f);
IPw = referenceElement.IPweights; ngauss = length(IPw);

%%Volume computations
% Jacobian
J11 = Nxi*Xe(:,1); J12 = Nxi*Xe(:,2); 
J21 = Neta*Xe(:,1); J22 = Neta*Xe(:,2); 
detJ = J11.*J22-J12.*J21;
%maybe we should use bsxfun instead of diagonal matrices...
dvolu = spdiags(referenceElement.IPweights.*detJ,0,ngauss,ngauss);
invJ11 = spdiags(J22./detJ,0,ngauss,ngauss);
invJ12 = spdiags(-J12./detJ,0,ngauss,ngauss);
invJ21 = spdiags(-J21./detJ,0,ngauss,ngauss);
invJ22 = spdiags(J11./detJ,0,ngauss,ngauss);
% xy-derivatives for approximation
Nx = invJ11*Nxi + invJ12*Neta;
Ny = invJ21*Nxi + invJ22*Neta;

%Computation of r.h.s. source term (analytical laplacian)
Xg = N*Xe;
sourceTerm = bodyForce(Xg); 
nuXg = viscosityMatrix(Xg);
advXg = advectionField(Xg);
fe = N'*(dvolu*sourceTerm);
fe = reshape(fe,2*nOfElementNodes,1);
%Basic elemental matrices
Me = N'*(dvolu*N); Cxe = N'*(dvolu*Nx); Cye = N'*(dvolu*Ny); 
%HDG elemental matrices
aux1=1:nOfElementNodes; 
aux2=aux1+nOfElementNodes; 
aux3=aux2+nOfElementNodes; 
aux4=aux3+nOfElementNodes;
ALL = zeros(4*nOfElementNodes,4*nOfElementNodes);
    ALL(aux1,aux1) = Me; 
    ALL(aux2,aux2) = Me; 
    ALL(aux3,aux3) = Me; 
    ALL(aux4,aux4) = Me; 
ALu = zeros(4*nOfElementNodes,2*nOfElementNodes);
    ALu(aux1,aux1) = Cxe'; 
    ALu(aux2,aux1) = Cye'; 
    ALu(aux3,aux2) = Cxe'; 
    ALu(aux4,aux2) = Cye';
AuL = zeros(2*nOfElementNodes,4*nOfElementNodes); 
%     AuL(aux1,aux1) = N'*dvolu*(-nuXg(:,1).*Nx + advXg(:,1).*N); 
%     AuL(aux1,aux2) = N'*dvolu*(-nuXg(:,1).*Ny + advXg(:,2).*N); 
%     AuL(aux1,aux3) = N'*dvolu*(-nuXg(:,2).*Nx); 
%     AuL(aux1,aux4) = N'*dvolu*(-nuXg(:,2).*Ny);
%     AuL(aux2,aux1) = N'*dvolu*(-nuXg(:,3).*Nx); 
%     AuL(aux2,aux2) = N'*dvolu*(-nuXg(:,3).*Ny); 
%     AuL(aux2,aux3) = N'*dvolu*(-nuXg(:,4).*Nx + advXg(:,1).*N); 
%     AuL(aux2,aux4) = N'*dvolu*(-nuXg(:,4).*Ny + advXg(:,2).*N);
% Version compatible with previous versions of Matlab (tested with matlab2012b)
    AuL(aux1,aux1) = N'*dvolu*(-repmat(nuXg(:,1),1,size(Nx,2)).*Nx + repmat(advXg(:,1),1,size(N,2)).*N); 
    AuL(aux1,aux2) = N'*dvolu*(-repmat(nuXg(:,1),1,size(Ny,2)).*Ny + repmat(advXg(:,2),1,size(N,2)).*N); 
    AuL(aux1,aux3) = N'*dvolu*(-repmat(nuXg(:,2),1,size(Nx,2)).*Nx); 
    AuL(aux1,aux4) = N'*dvolu*(-repmat(nuXg(:,2),1,size(Ny,2)).*Ny);
    AuL(aux2,aux1) = N'*dvolu*(-repmat(nuXg(:,3),1,size(Nx,2)).*Nx); 
    AuL(aux2,aux2) = N'*dvolu*(-repmat(nuXg(:,3),1,size(Ny,2)).*Ny); 
    AuL(aux2,aux3) = N'*dvolu*(-repmat(nuXg(:,4),1,size(Nx,2)).*Nx + repmat(advXg(:,1),1,size(N,2)).*N); 
    AuL(aux2,aux4) = N'*dvolu*(-repmat(nuXg(:,4),1,size(Ny,2)).*Ny + repmat(advXg(:,2),1,size(N,2)).*N);
Aup = [Cxe;Cye]; 
Apu = Aup';

%% Faces computations
ALl = zeros(4*nOfElementNodes,3*2*nOfFaceNodes);
AlL = zeros(3*2*nOfFaceNodes,4*nOfElementNodes);
Auu = zeros(2*nOfElementNodes,2*nOfElementNodes);
Aul = zeros(2*nOfElementNodes,3*2*nOfFaceNodes);
Apl = zeros(nOfElementNodes,3*2*nOfFaceNodes);
All = zeros(3*2*nOfFaceNodes,3*2*nOfFaceNodes);
Arp = zeros(1,nOfElementNodes);
Arl = zeros(1,3*2*nOfFaceNodes);
lengthBoundaryElement=0;
for iface = 1:nOfFaces
    Xf = Xe(faceNodes(iface,:),:); % Nodes in the face
    Xgf = N1d*Xf;
    nuXgf = viscosityMatrix(Xgf);
    nuFace = nuXgf(1,1);
    advXgf = advectionField(Xgf);
    dxdxi = Nx1d*Xf(:,1); dydxi = Nx1d*Xf(:,2);
    dxdxiNorm = sqrt(dxdxi.^2+dydxi.^2);
    dline = dxdxiNorm.*IPw_f';
    nx = dydxi./dxdxiNorm; ny=-dxdxi./dxdxiNorm;
    % Definition of the stabilization parameter
    mutau_f = nuFace/2.*tau(iface);    
    %Face basic matrices
    Mf = N1d'*(spdiags(dline,0,ngf,ngf)*N1d);
    Mfnx = N1d'*(spdiags(dline.*nx,0,ngf,ngf)*N1d);
    Mfny = N1d'*(spdiags(dline.*ny,0,ngf,ngf)*N1d);
    intN = dline'*N1d;
    intNnx = (dline.*nx)'*N1d;
    intNny = (dline.*ny)'*N1d;
    %HDG face matrices
    nodes1 = faceNodes(iface,:);
     nodes2 = nodes1+nOfElementNodes; nodes3=nodes2+nOfElementNodes; nodes4=nodes3+nOfElementNodes;
    ind_face1 = (iface-1)*2*nOfFaceNodes + (1:nOfFaceNodes); 
     ind_face2 = ind_face1 + nOfFaceNodes;
    %
    Auu(nodes1,nodes1) = Auu(nodes1,nodes1) + mutau_f*Mf; 
    Auu(nodes2,nodes2) = Auu(nodes2,nodes2) + mutau_f*Mf; 
    %
    Aul(nodes1,ind_face1) = Aul(nodes1,ind_face1) - mutau_f*Mf; 
    Aul(nodes2,ind_face2) = Aul(nodes2,ind_face2) - mutau_f*Mf;
    %
    ALl(nodes1,ind_face1) = ALl(nodes1,ind_face1) - Mfnx;
    ALl(nodes2,ind_face1) = ALl(nodes2,ind_face1) - Mfny;
    ALl(nodes3,ind_face2) = ALl(nodes3,ind_face2) - Mfnx;
    ALl(nodes4,ind_face2) = ALl(nodes4,ind_face2) - Mfny;    
    %
%     AlL(ind_face1,nodes1) = AlL(ind_face1,nodes1) - N1d'*(nuXgf(:,1).*spdiags(dline.*nx,0,ngf,ngf)*N1d);
%     AlL(ind_face1,nodes2) = AlL(ind_face1,nodes2) - N1d'*(nuXgf(:,1).*spdiags(dline.*ny,0,ngf,ngf)*N1d);
%     AlL(ind_face1,nodes3) = AlL(ind_face1,nodes3) - N1d'*(nuXgf(:,2).*spdiags(dline.*nx,0,ngf,ngf)*N1d);
%     AlL(ind_face1,nodes4) = AlL(ind_face1,nodes4) - N1d'*(nuXgf(:,2).*spdiags(dline.*ny,0,ngf,ngf)*N1d);    
%     AlL(ind_face2,nodes1) = AlL(ind_face2,nodes1) - N1d'*(nuXgf(:,3).*spdiags(dline.*nx,0,ngf,ngf)*N1d);
%     AlL(ind_face2,nodes2) = AlL(ind_face2,nodes2) - N1d'*(nuXgf(:,3).*spdiags(dline.*ny,0,ngf,ngf)*N1d);
%     AlL(ind_face2,nodes3) = AlL(ind_face2,nodes3) - N1d'*(nuXgf(:,4).*spdiags(dline.*nx,0,ngf,ngf)*N1d);
%     AlL(ind_face2,nodes4) = AlL(ind_face2,nodes4) - N1d'*(nuXgf(:,4).*spdiags(dline.*ny,0,ngf,ngf)*N1d);         
% Version compatible with previous versions of Matlab (tested with matlab2012b)
    AlL(ind_face1,nodes1) = AlL(ind_face1,nodes1) - N1d'*(repmat(nuXgf(:,1),1,ngf).*spdiags(dline.*nx,0,ngf,ngf)*N1d);
    AlL(ind_face1,nodes2) = AlL(ind_face1,nodes2) - N1d'*(repmat(nuXgf(:,1),1,ngf).*spdiags(dline.*ny,0,ngf,ngf)*N1d);
    AlL(ind_face1,nodes3) = AlL(ind_face1,nodes3) - N1d'*(repmat(nuXgf(:,2),1,ngf).*spdiags(dline.*nx,0,ngf,ngf)*N1d);
    AlL(ind_face1,nodes4) = AlL(ind_face1,nodes4) - N1d'*(repmat(nuXgf(:,2),1,ngf).*spdiags(dline.*ny,0,ngf,ngf)*N1d);    
    AlL(ind_face2,nodes1) = AlL(ind_face2,nodes1) - N1d'*(repmat(nuXgf(:,3),1,ngf).*spdiags(dline.*nx,0,ngf,ngf)*N1d);
    AlL(ind_face2,nodes2) = AlL(ind_face2,nodes2) - N1d'*(repmat(nuXgf(:,3),1,ngf).*spdiags(dline.*ny,0,ngf,ngf)*N1d);
    AlL(ind_face2,nodes3) = AlL(ind_face2,nodes3) - N1d'*(repmat(nuXgf(:,4),1,ngf).*spdiags(dline.*nx,0,ngf,ngf)*N1d);
    AlL(ind_face2,nodes4) = AlL(ind_face2,nodes4) - N1d'*(repmat(nuXgf(:,4),1,ngf).*spdiags(dline.*ny,0,ngf,ngf)*N1d);         
    %
    Apl(nodes1,ind_face1) = Apl(nodes1,ind_face1) - Mfnx; 
    Apl(nodes1,ind_face2) = Apl(nodes1,ind_face2) - Mfny;
    %
    Arp(nodes1) = Arp(nodes1) + intN;
    %
    All(ind_face1,ind_face1) = All(ind_face1,ind_face1) - mutau_f*Mf;
    All(ind_face2,ind_face2) = All(ind_face2,ind_face2) - mutau_f*Mf;
    if(isInterior(iface))
%         All(ind_face1,ind_face1) = All(ind_face1,ind_face1) + N1d'*(advXgf(:,1).*spdiags(dline.*nx,0,ngf,ngf) + advXgf(:,2).*spdiags(dline.*ny,0,ngf,ngf))*N1d;
%         All(ind_face2,ind_face2) = All(ind_face2,ind_face2) + N1d'*(advXgf(:,1).*spdiags(dline.*nx,0,ngf,ngf) + advXgf(:,2).*spdiags(dline.*ny,0,ngf,ngf))*N1d;
% Version compatible with previous versions of Matlab (tested with matlab2012b)
        All(ind_face1,ind_face1) = All(ind_face1,ind_face1) + N1d'*(repmat(advXgf(:,1),1,ngf).*spdiags(dline.*nx,0,ngf,ngf) + repmat(advXgf(:,2),1,ngf).*spdiags(dline.*ny,0,ngf,ngf))*N1d;
        All(ind_face2,ind_face2) = All(ind_face2,ind_face2) + N1d'*(repmat(advXgf(:,1),1,ngf).*spdiags(dline.*nx,0,ngf,ngf) + repmat(advXgf(:,2),1,ngf).*spdiags(dline.*ny,0,ngf,ngf))*N1d;
    end
    %
    Arl([ind_face1,ind_face2]) = Arl([ind_face1,ind_face2])+[intNnx,intNny];
    lengthBoundaryElement = lengthBoundaryElement + sum(dline);
end
Arp=Arp/lengthBoundaryElement;
Alp=-Apl';
Alu=-Aul';

% Elemental mapping ( system A*[u L p lambda]=b - B*Lambda_e )
% lambda is the Lagrange multiplier to impose the constraint
% Lambda_e contains the trace of u at the faces AND the density mean (rho)
A = zeros(7*nOfElementNodes+1);
indu = 1:2*nOfElementNodes; indL=2*nOfElementNodes + [1:4*nOfElementNodes]; 
indp = 6*nOfElementNodes + [1:nOfElementNodes];
A(indu,[indu,indL,indp]) = [Auu AuL Aup];
A(indL,[indu,indL]) = [ALu ALL];
A(indp,indu) = Apu;
A(end,indp) = Arp; A(indp,end) = Arp';
b = zeros(7*nOfElementNodes+1,1);
b(indu) = fe;
Br = [zeros(7*nOfElementNodes,1) ; 1];
Bl = [Aul ; ALl ; Apl ; zeros(1,6*nOfFaceNodes)];
B = [Bl,Br]; %Lambda_e contains the face values of u and the rho value

LocalSolverMatElem=-A\B; LocalSolverMatElem=LocalSolverMatElem(1:end-1,:);
LocalSolverVecElem=A\b; LocalSolverVecElem=LocalSolverVecElem(1:end-1);

%save matrices A Aul Aql All fe sourceTerm

